
;
; written by ville helin <vhelin@iki.fi> in 2004
;

.include "defines.i"

;
; sm4x4's variables in high ram
;

.define SM4x4_FRAMEP $80

;
; sm4x4 - chunky mode (80x72)
;

.bank 0 slot 0
.org $150
.section "sm4x4" semifree

;
; sm4x4 init - build the 4x1 tiles
;

SM4x4_INIT:
	ld	b, 128
	ld	hl, $8800

-	ld	a, b
	and	%01010101	; low bits
	ld	d, a
	sla	a
	or	d		; 4x1
	ld	d, a		; d ~ low 4x1

	ld	a, b
	and	%10101010	; high bits
	ld	e, a
	sra	a
	or	e		; 4x1
*	ld	e, a		; e ~ high 4x1

	.rept 8			; create the tile
	ld	(hl), d
	inc	hl
	ld	(hl+), a
	.endr

	inc	b
	ld	a, b
	cp	128
	jr	nz, -

	; touch the display settings...
	ld	hl, $ff40	; $8800+
	res	4, (hl)
	res	3, (hl)
	set	0, (hl)
	xor	a
	ld	hl, $ff42
	ld	(hl), a
	inc	hl
	ld	(hl), a

	; init the lcd irq
	ld	hl, IRQ_LCD
	ld	a, $C3			; jp.
	ld	(hl+), a
	ld	a, SM4x4_SKIP&$ff	; low.
	ld	(hl+), a
	ld	a, SM4x4_SKIP>>8	; high.
	ld	(hl), a

	ret

;
; sm4x4 vbi
;

SM4x4_VBI:
	push	af
	push	hl
	push	bc

	; init the frame pointer
	ld	hl, DAT_LOGO1
	ld	a, l
	ld	($ff00+SM4x4_FRAMEP+0), a
	ld	a, h
	ld	($ff00+SM4x4_FRAMEP+1), a

	; set the lcd interrupt
	ld	hl, IRQ_LCD+1
	ld	a, SM4x4_SKIP&$ff	; low.
	ld	(hl+), a
	ld	a, SM4x4_SKIP>>8	; high.
	ld	(hl), a

	; copy the first 20 tiles directly into vram
	ld	a, ($ff00+SM4x4_FRAMEP+0)
	ld	l, a
	ld	a, ($ff00+SM4x4_FRAMEP+1)
	ld	h, a
	ld	bc, $9800

	.rept 19
	ld	a, (hl+)
	ld	(bc), a
	inc	c		; inc c is enough as c starts from $00.
	.endr
	ld	a, (hl+)
	ld	(bc), a

	; save the pointer
	ld	a, l
	ld	($ff00+SM4x4_FRAMEP+0), a
	ld	a, h
	ld	($ff00+SM4x4_FRAMEP+1), a

	pop	bc
	pop	hl
	pop	af

	reti

;
; sm4x4 scanline skipper interrupt
;

SM4x4_SKIP:
	push	af
	push	hl

	; set the next hblank irq to copy tiles
	ld	hl, IRQ_LCD+1
	ld	a, ($ff00+SM4x4_FRAMEP+0) ; low.
	ld	(hl+), a
	add	76			  ; the size of the hblank irq routine.
	ld	($ff00+SM4x4_FRAMEP+0), a
	ld	a, ($ff00+SM4x4_FRAMEP+1) ; high.
	ld	(hl), a
	adc	0
	ld	($ff00+SM4x4_FRAMEP+1), a

	pop	hl
	pop	af

	reti

.ends
